function setup_attach()
    vim.api.nvim_create_autocmd('LspAttach', {
        group = vim.api.nvim_create_augroup('UserLspConfig', {}),
        callback = function (ev)
            -- local lspconfig  = require('lspconfig')
            --
            vim.bo[ev.buf].omnifunc = 'v:lua.vim.lsp.omnifunc'

            local opts = { buffer = ev.buf }
            vim.keymap.set('n', 'gD', vim.lsp.buf.declaration, opts)
            vim.keymap.set('n', 'gd', vim.lsp.buf.definition, opts)
            vim.keymap.set('n', 'K', vim.lsp.buf.hover, opts)
            vim.keymap.set('n', 'gi', vim.lsp.buf.implementation, opts)
            vim.keymap.set('n', '<C-k>', vim.lsp.buf.signature_help, opts)
            -- vim.keymap.set('n', '<space>wa', vim.lsp.buf.add_workspace_folder, opts)
            -- vim.keymap.set('n', '<space>wr', vim.lsp.buf.remove_workspace_folder, opts)
            -- vim.keymap.set('n', '<space>D', vim.lsp.buf.type_definition, opts)
            -- vim.keymap.set('n', '<space>rn', vim.lsp.buf.rename, opts)
            vim.keymap.set({ 'n', 'v' }, '<leader>ca', vim.lsp.buf.code_action, opts)
            vim.keymap.set('n', 'gr', vim.lsp.buf.references, opts)
            vim.keymap.set('n', 'gf', function()
                vim.lsp.buf.format { async = true }
            end, opts)
        end,
    })
end

function setup_cmp() 
    local cmp = require('cmp')
    cmp.setup({
        snippet = {
            expand = function(args)
                require('luasnip').lsp_expand(args.body)
            end,
        },
        window = {
            -- completion = cmp.config.window.bordered(),
            -- documentation = cmp.config.window.bordered(),
        },
        mapping = cmp.mapping.preset.insert({
            ['<C-b>'] = cmp.mapping.scroll_docs(-4),
            ['<C-f>'] = cmp.mapping.scroll_docs(4),
            ['<C-Space>'] = cmp.mapping.complete(),
            ['<C-e>'] = cmp.mapping.abort(),
            ['<CR>'] = cmp.mapping.confirm({ select = true }), -- Accept currently selected item. Set `select` to `false` to only confirm explicitly selected items.
        }),
        sources = cmp.config.sources({
            { name = 'nvim_lsp' },
            { name = 'luasnip' },
        }, {
            { name = 'buffer' },
        })
    })

    cmp.setup.cmdline(':', {
        mapping = cmp.mapping.preset.cmdline(),
        sources = cmp.config.sources({
            { name = 'path' }
        }, {
            { name = 'cmdline' }
        })
    })
end

return {

    {
        "j-hui/fidget.nvim",
        name = "fidget",
        opts = {}
    },
    
    {
        "neovim/nvim-lspconfig",
        dependencies = {
            'neovim/nvim-lspconfig',
            'hrsh7th/cmp-nvim-lsp',
            'hrsh7th/cmp-buffer',
            'hrsh7th/cmp-path',
            'hrsh7th/cmp-cmdline',
            'hrsh7th/nvim-cmp',
            'saadparwaiz1/cmp_luasnip',
            "ray-x/lsp_signature.nvim",
            'luasnip',
            'fidget',
        },
        config = function()
           local lspconfig  = require('lspconfig')

           vim.diagnostic.config({
               virtual_text = {
                   prefix = "👈",
               },
               signs = { 
                   text = { 
                       [vim.diagnostic.severity.ERROR] = "🆒", 
                       [vim.diagnostic.severity.WARN] = "😞", 
                       [vim.diagnostic.severity.HINT] = "🆘",
                       [vim.diagnostic.severity.INFO] = "🆓", 
                       CodeAction = "🈁", 
                   } 
               },
           })

           -- Check out trouble
           vim.keymap.set('n', '[d', vim.diagnostic.goto_prev)
           vim.keymap.set('n', ']d', vim.diagnostic.goto_next)

           setup_attach()
           setup_cmp()
        
           local capabilities = vim.tbl_deep_extend(
                "force",
                {},
                vim.lsp.protocol.make_client_capabilities(),
                require('cmp_nvim_lsp').default_capabilities()
           )
        
           local def_set_servers = { "tsserver", "svelte", "zls", "gopls" }

           for _, server_name in ipairs(def_set_servers) do
               lspconfig[server_name].setup({
                   capabilities = capabilities,
               })
           end

           lspconfig.elixirls.setup({
               cmd = { "elixir-ls" },
               capabilities = capabilities,
           })

           require("lsp_signature").on_attach({bind = true})
        end,
    },

}
